package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBasePriceStrategyCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.SpaceInstitutionDetailEnt;
import pps.core.space.entity.SpaceInstitutionWellheadEnt;
import pps.core.space.entity.SpaceOptimizeShortDurationEnt;
import pps.core.space.entity.SpaceOptimizeShortWellheadEnt;
import pps.core.space.mapper.SpaceInstitutionDetailMapper;
import pps.core.space.mapper.SpaceInstitutionWellheadMapper;
import pps.core.space.mapper.SpaceOptimizeShortDurationMapper;
import pps.core.space.mapper.SpaceOptimizeShortWellheadMapper;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadInput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 间开优化对外提供接口服务
 *
 * @author ZWT
 * @date 2024/04/08 14:21
 */
@XService
@XText("间开优化对外提供接口服务")
public class SpaceOptimizeApiService extends SpaceOptimizeBaseService {

    @XApiGet(anonymous = true)
    @XText("极短期间开优化--计算一个率")
    public XServiceResult getOptimization(XContext context) {
        DateTime of = DateTime.of("2024-04-02", "yyyy-MM-dd");
        //先查并网线路
        IBasePowerLineCloudService lineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineOutputXListResult = lineCloudService.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .gridTypeKey("1")
                .build());
        //市电峰谷
        IBasePriceStrategyCloudService strategyCloudService = context.getBean(IBasePriceStrategyCloudService.class);
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
        SpaceOptimizeShortDurationMapper shortDurationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        SpaceInstitutionDetailEnt detailEnt;
        List<SpaceInstitutionWellheadEnt> wellheadList;
        XListResult<DynamicQueryBasePowerLinePlantViewOutput> plantResult;
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList;
        XListResult<GetBasePriceStrategyDetailOutput> strategyResult;
        List<GetBasePriceStrategyDetailOutput> strategyList;
        List<SpaceOptimizeShortDurationEnt> shortDurationList;
        List<DynamicQueryPlantPredictedPowerOutput> generationHourList;
        List<DynamicQueryBasePowerLineOutput> lineList = lineOutputXListResult.getResult();
        for (DynamicQueryBasePowerLineOutput line : lineList) {
            //查对应间开制度
            detailEnt = detailMapper.selectOne(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                    .eq(SpaceInstitutionDetailEnt::getLineId, line.getId())
            );
            if (ObjectUtil.isEmpty(detailEnt)) {
                continue;
            }
            //查间开关联井口(间抽井,小间开)
            wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceInstitutionWellheadEnt::getInstitutionId, detailEnt.getId())
                    .eq(SpaceInstitutionWellheadEnt::getRunTypeKey, BusinessConstant.INTERVAL_PUMPING_WELL)
                    .eq(SpaceInstitutionWellheadEnt::getIntervalTypeKey, "0")
                    .orderByAsc(SpaceInstitutionWellheadEnt::getStartSeq)
            );
            if (CollUtil.isEmpty(wellheadList)) {
                continue;
            }
            //查线路下所有电站
            plantResult = lineCloudService.getPowerLinePlantViewList(context, DynamicQueryBasePowerLinePlantInput.builder()
                    .lineId(line.getId())
                    .build());
            plantList = plantResult.getResult();
            if (CollUtil.isEmpty(plantList)) {
                continue;
            }
            //查线路关联市电峰谷
            strategyResult = strategyCloudService.queryStrategyDetailList(context, GetBasePriceStrategyDetailInput.builder()
                    .lineId(line.getId())
                    .periodTypeKey("LOW")
                    .strategyMonth("4")
                    .build());
            strategyList = strategyResult.getResult();
            if (CollUtil.isEmpty(strategyList)) {
                continue;
            }
            //查间开优化时间段
            shortDurationList = shortDurationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, of)
                    .in(SpaceOptimizeShortDurationEnt::getWellheadId, wellheadList.stream()
                            .map(SpaceInstitutionWellheadEnt::getWellheadId)
                            .collect(Collectors.toList())
                    )
                    .orderByAsc(SpaceOptimizeShortDurationEnt::getOpenWellTime)
            );
            if (CollUtil.isEmpty(shortDurationList)) {
                continue;
            }
            //查当日预测发电时间段
            generationHourList = this.getAveragePowerGenerationHourList(context, DynamicQueryPlantPredictedPowerInput.builder()
                    .plantIds(plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getPlantId)
                            .collect(Collectors.toList())
                    )
                    .startTime(of.toString())
                    .endTime(DateUtil.offsetDay(of, 1).toString())
                    .dateType(BusinessConstant.ONE)
                    .build());

        }
        return XServiceResult.OK;
    }


    /**
     * 每日间开优化结果
     *
     * @param context 上下文
     * @return {@link XListResult}<{@link QuerySpaceOptimizeShortWellheadOutput}>
     */
    @XApiPost(anonymous = true)
    @XText("极短期间开优化--每日间开优化结果")
    public XListResult<QuerySpaceOptimizeShortWellheadOutput> getDailyOptimization(XContext context, QuerySpaceOptimizeShortWellheadInput input) {
        Date queryDate = input.getQueryDate();
        if (Objects.isNull(queryDate)) {
            queryDate = DateUtil.date();
        }
        DateTime beginOfDay = DateUtil.beginOfDay(queryDate);
        SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .eq(SpaceOptimizeShortWellheadEnt::getOptimizeDate, beginOfDay)
                .orderByAsc(SpaceOptimizeShortWellheadEnt::getStartSeq)
        );
        if (CollUtil.isEmpty(wellheadList)) {
            return XListResult.success(Collections.emptyList());
        }
        SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        List<SpaceOptimizeShortDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceOptimizeShortDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, beginOfDay)
                .in(SpaceOptimizeShortDurationEnt::getWellheadId, wellheadList.stream()
                        .map(SpaceOptimizeShortWellheadEnt::getWellheadId)
                        .collect(Collectors.toList())
                )
                .orderByAsc(SpaceOptimizeShortDurationEnt::getOpenWellTime)
        );
        List<QuerySpaceOptimizeShortWellheadOutput> outputs = new ArrayList<>(wellheadList.size());
        List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
        Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
        for (SpaceOptimizeShortWellheadEnt wellheadEnt : wellheadList) {
            outputs.add(
                    QuerySpaceOptimizeShortWellheadOutput.builder()
                            .wellNumber(wellheadEnt.getWellNumber())
                            .optimizeList(ServiceUtil.getAllDateDuration(collect.get(wellheadEnt.getWellheadId())))
                            .build()
            );
        }
        return XListResult.success(outputs);
    }
}
